{****************************************************************************
**                                                                         **
**                         Util module for PSSDK                           **
**              Copyright (c) 2004 microOLAP Technologies LLC,             **
**                      Halturin A.P., Naumov D.A.                         **
**                              Source File                                **
**                                                                         **
****************************************************************************}
unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, HNSyncList, HNLogConsole, ExtCtrls,
  //----------------------------------------------------------------------
  HNWorkThread,
  HNPsSdkDef, HNPsSdkUtil, HNPSManager, HNAdapterConfig, HNUserFilter,
  HNAdapter;

//-----------------------------------------------------------------------------
// HN_WM_THREAD_EXIT - our window message for informing of the working thread 
//                     exit
Const
  HN_WM_THREAD_EXIT = WM_USER + 1;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    cbNetCardList: TComboBox;
    About_Btn: TButton;
    Init_Btn: TButton;
    Open_Btn: TButton;
    Close_Btn: TButton;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    UserFilterFileName: TEdit;
    EnableFilter_Btn: TButton;
    DisableFilter_Btn: TButton;
    FilterOpen_Btn: TButton;
    GroupBox3: TGroupBox;
    chkDumpLimit: TCheckBox;
    DumpLimit: TEdit;
    chkDumpToFile: TCheckBox;
    chkDumpToConsole: TCheckBox;
    Label2: TLabel;
    DumpFileName: TEdit;
    DumpFile_Btn: TButton;
    GroupBox4: TGroupBox;
    GroupBox5: TGroupBox;
    Label3: TLabel;
    NdisTranWithoutErr: TPanel;
    Label4: TLabel;
    NdisRecvWithoutErr: TPanel;
    NdisTranWithErr: TPanel;
    Label5: TLabel;
    NdisRecvWithErr: TPanel;
    Label6: TLabel;
    NdisMissed: TPanel;
    Label7: TLabel;
    Label8: TLabel;
    BpfReceive: TPanel;
    Label9: TLabel;
    BpfAccept: TPanel;
    Label10: TLabel;
    BpfReject: TPanel;
    Label11: TLabel;
    BpfProcess: TPanel;
    HNPSManager: THNPSManager;
    HNAdapterConfig: THNAdapterConfig;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    HNUserFilter: THNUserFilter;
    HNAdapter: THNAdapter;
    Timer: TTimer;
    chkUseFastBpf: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure About_BtnClick(Sender: TObject);
    procedure Init_BtnClick(Sender: TObject);
    procedure HNPSManagerConfigChange(Sender: TObject; hConfig: Pointer;
      ChangeType: Cardinal);
    procedure FilterOpen_BtnClick(Sender: TObject);
    procedure DumpFile_BtnClick(Sender: TObject);
    procedure EnableFilter_BtnClick(Sender: TObject);
    procedure DisableFilter_BtnClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure chkDumpLimitClick(Sender: TObject);
    procedure Open_BtnClick(Sender: TObject);
    procedure Close_BtnClick(Sender: TObject);
    procedure HNAdapterPacketRecv(Sender: TObject; pPacket: Pointer;
      PacketSize: Integer; pNumberID: PULargeInteger;
      pTimeStamp: PLargeInteger);
    procedure chkUseFastBpfClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Thread  : THNWorkThread;

  public
    procedure GetStat(IsAsync : boolean);
    procedure Stop();
    procedure WndProc(var Msg: TMessage); override;
  end;
var
  MainForm  : TMainForm;
  HNConsole : THNLogConsole;

  hLogFile      : THandle;
  UseLimit      : BOOL;
  LimitCount    : DWORD;
  DumpToConsole : BOOL;
  DumpToFile    : BOOL;

implementation

{$R *.DFM}

//---------------------------------------------------------------------------
// FormCreate
//---------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
 begin
   HNConsole := THNLogConsole.Create();
   Thread    := THNWorkThread.Create();

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Customizing the console window. Size 80300 characters. 
   HNConsole.SetWinSize(80,300);
   HNConsole.SetTitle('DumpDemo log console');

   cbNetCardList.ItemIndex := 0;

 end;

//---------------------------------------------------------------------------
// FormDestroy - the handler for destroying the form
//---------------------------------------------------------------------------
procedure TMainForm.FormDestroy(Sender: TObject);
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Stop receiving packets
   Close_BtnClick(nil);

   Thread.Free();
   HNConsole.Free();
 end;

//-----------------------------------------------------------------------------
// OnAboutBtn - the "About" button is pressed
//-----------------------------------------------------------------------------
procedure TMainForm.About_BtnClick(Sender: TObject);
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Show the library build & version
   ShowMessage('PSSDK v.' + HNPSManager.Version + ' Build ' + HNPSManager.Build + #10#13 +'microOLAP Technologies');
 end;

//-----------------------------------------------------------------------------
// Init_BtnClick - the manager initialization button is pressed
//-----------------------------------------------------------------------------
procedure TMainForm.Init_BtnClick(Sender: TObject);
 var
   Res  : DWORD;
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Clear the adapter list in ComboBox
   cbNetCardList.Clear();

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Initialize the PSSDK library
   Res := HNPSManager.Initialize();
   if(Res <> HNERR_OK) then
    begin
      HNGetErrorBox(Res);
      Exit;
    end;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Show the first adapter in ComboBox
   cbNetCardList.ItemIndex := 0;

   Init_Btn.Enabled := False;
 end;

//----------------------------------------------------------------------------------
// OnConfigChange - a handler for the OnConfigChange event of the HNPSManager object
//----------------------------------------------------------------------------------
procedure TMainForm.HNPSManagerConfigChange(Sender: TObject; hConfig: Pointer; ChangeType: Cardinal);
 begin

   HNAdapterConfig.Handle := hConfig;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Handles the event on adding new configurations 
   // found by the manager
   if(ChangeType = acCreate) then
    begin
      //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      // Adding to the list only adapters of the "Ethernet" type
      if(HNAdapterConfig.AdapterType = atEthernet) then
       begin
         cbNetCardList.Items.AddObject(HNAdapterConfig.AdapterDescription,hConfig);
       end;
    end;
 end;

//------------------------------------------------------------------------------
// FilterOpen_BtnClick - the button for selecting a user BPF filter file is presssed
//------------------------------------------------------------------------------
procedure TMainForm.FilterOpen_BtnClick(Sender: TObject);
 begin
   OpenDialog.Title    := 'User BPF filter';
   OpenDialog.Filter   := 'User BPF filters (*.bpf)|*.bpf|All Files (*.*)|*.*||';
   OpenDialog.FileName := UserFilterFileName.Text;

   if(not OpenDialog.Execute()) then Exit;

   UserFilterFileName.Text := OpenDialog.FileName;
 end;

//---------------------------------------------------------------------------
// DumpFile_BtnClick - the button for selecting a dump file is pressed
//---------------------------------------------------------------------------
procedure TMainForm.DumpFile_BtnClick(Sender: TObject);
 begin
   SaveDialog.Title    := 'Dump file';
   SaveDialog.Filter   := 'Text files (*.txt)|*.txt|All Files (*.*)|*.*||';
   SaveDialog.FileName := DumpFileName.Text;

   if(not SaveDialog.Execute()) then Exit;

   DumpFileName.Text := SaveDialog.FileName;
 end;

//---------------------------------------------------------------------------
// EnableFilter_BtnClick - the "Enable filter" button is pressed
//---------------------------------------------------------------------------
procedure TMainForm.EnableFilter_BtnClick(Sender: TObject);
 var
   Res : DWORD; 
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Load the user BPF filter from the file
   Res := HNUserFilter.LoadFromFile(PChar(UserFilterFileName.Text));
   if(Res <> HNERR_OK) then
    begin
      HNGetErrorBox(Res);
      Exit;
    end;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Set the user BPF filter to the adapter
   HNAdapter.UserFilter := HNUserFilter.Handle;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Activate the user BDF filter
   HNAdapter.UserFilterActive := True;

   HNConsole.printf('User settable BPF filter %s enabled.'#13#10#13#10,[PChar(UserFilterFileName.Text)]);
 end;

//---------------------------------------------------------------------------
// OnDisableFilterBtn - the "Disable filter" button is pressed
//---------------------------------------------------------------------------
procedure TMainForm.DisableFilter_BtnClick(Sender: TObject);
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Deactivate the user BPF filter
   HNAdapter.UserFilterActive := False;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Remove the user BPF filter from the adapter
   HNAdapter.UserFilter := nil;

   HNConsole.printf('User settable BPF filter disabled ...'#13#10#13#10,[]);
 end;

//---------------------------------------------------------------------------
// GetStat - receive the adapter statistics synchronously and asynchronously 
//---------------------------------------------------------------------------
procedure TMainForm.GetStat(IsAsync : boolean);
 var
   Stat  : ULARGE_INTEGER;
 begin
   if(IsAsync) then
    begin
      //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      // Refresh the adapter statistics asynchronously 
      HNAdapter.Get_AsyncStatistics(stNdisTranWithoutErr);
      HNAdapter.Get_AsyncStatistics(stNdisRecvWithoutErr);
      HNAdapter.Get_AsyncStatistics(stNdisTranWithErr);
      HNAdapter.Get_AsyncStatistics(stNdisRecvWithErr);
      HNAdapter.Get_AsyncStatistics(stNdisMissed);

      //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      // Refresh the user BPF filter statistics asynchronously 
      HNAdapter.Get_AsyncStatistics(stBpfReceive);
      HNAdapter.Get_AsyncStatistics(stBpfAccept);
      HNAdapter.Get_AsyncStatistics(stBpfReject);
      HNAdapter.Get_AsyncStatistics(stBpfProcess);
    end
   else
    begin
      //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      // Refresh the adapter statistics synchronously
      HNAdapter.UpdateAdapterStatistics();

      //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      // Refresh the user BPF filter statistics synchronously
      HNAdapter.UpdateUserStatistics();
    end;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Display the adapter statistics
   Stat.LowPart := HNAdapter.Get_Missed(@Stat.HighPart);
   NdisMissed.Caption := IntToStr(Stat.QuadPart);

   Stat.LowPart := HNAdapter.Get_RecvWithErr(@Stat.HighPart);
   NdisRecvWithErr.Caption := IntToStr(Stat.QuadPart);

   Stat.LowPart := HNAdapter.Get_RecvWithoutErr(@Stat.HighPart);
   NdisRecvWithoutErr.Caption := IntToStr(Stat.QuadPart);

   Stat.LowPart := HNAdapter.Get_TranWithErr(@Stat.HighPart);
   NdisTranWithErr.Caption := IntToStr(Stat.QuadPart);

   Stat.LowPart := HNAdapter.Get_TranWithoutErr(@Stat.HighPart);
   NdisTranWithoutErr.Caption := IntToStr(Stat.QuadPart);

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Display the user BPF filter statistics
   Stat.LowPart := HNAdapter.Get_AcceptCount(@Stat.HighPart);
   BpfAccept.Caption := IntToStr(Stat.QuadPart);

   Stat.LowPart := HNAdapter.Get_ProcessCount(@Stat.HighPart);
   BpfProcess.Caption := IntToStr(Stat.QuadPart);

   Stat.LowPart := HNAdapter.Get_RecvCount(@Stat.HighPart);
   BpfReceive.Caption := IntToStr(Stat.QuadPart);

   Stat.LowPart := HNAdapter.Get_RejectCount(@Stat.HighPart);
   BpfReject.Caption := IntToStr(Stat.QuadPart);
 end;

//---------------------------------------------------------------------------
// TimerTimer - WM_TIMER message handler
//---------------------------------------------------------------------------
procedure TMainForm.TimerTimer(Sender: TObject);
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Refresh the statistics asynchronously
   GetStat(True);
 end;

//---------------------------------------------------------------------------
// OnDumpLimitChk - the "Use dump limit" CheckBox state is changed
//---------------------------------------------------------------------------
procedure TMainForm.chkDumpLimitClick(Sender: TObject);
 begin
   DumpLimit.Enabled := not chkDumpLimit.Checked;
 end;

//-----------------------------------------------------------------------------
// Open_BtnClick - the adapter open button is pressed
//-----------------------------------------------------------------------------
procedure TMainForm.Open_BtnClick(Sender: TObject);
 var
   Res  : DWORD;
 begin
   LimitCount    := StrToint(DumpLimit.Text);
   UseLimit      := chkDumpLimit.Checked;
   DumpToConsole := chkDumpToConsole.Checked;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Open the console if it is not opened yet
   if(not HNConsole.IsOpen()) then
    begin
      HNConsole.Open(SW_MAXIMIZE);
      HNConsole.printf('DumpDemo log console started ...'#13#10#13#10,[]);
    end;
   HNConsole.printf('############################################################################'#13#10#13#10,[]);

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Close the adapter and stop the packet handling thread, if the adapter was opened
   if(HNAdapter.IsOpened()) then
    begin
      HNAdapter.CloseAdapter(); // Close the network adapter
      Thread.StopAndWait();     // Stop the packet handling thread
      Timer.Enabled := False;   // Stop the statistics timer
    end;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Define the configuration selected in ComboBox by the user and receive
   // its handler
   // Set the selected configuration to the adapter
   if(cbNetCardList.ItemIndex = -1) then Exit;
   HNAdapter.ConfigHandle := cbNetCardList.Items.Objects[cbNetCardList.ItemIndex];

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Open the log file
   if(chkDumpToFile.Checked) then
    begin
     hLogFile := CreateFile(PChar(DumpFileName.Text),GENERIC_READ or GENERIC_WRITE,0, nil, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
     if(hLogFile = 0) then
      begin
        Application.MessageBox('Error opening dump file.','Error',IDOK);
        Exit;
      end;
    end;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Open the network adapter
   Res := HNAdapter.OpenAdapter();
   if(Res <> HNERR_OK) then  
    begin
      if(DumpToFile) then
       begin
         CloseHandle(hLogFile);
         hLogFile := 0;
       end;
      HNGetErrorBox(Res);
      Exit;
    end;

   Thread.ClearBusyList();

   DumpToFile := chkDumpToFile.Checked;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Run the packet handling thread
   Thread.Start(THREAD_PRIORITY_NORMAL);

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Run the statistics timer, firing one time per second
   Timer.Enabled := True;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Set the MAC filter for receiving all packets.
   // Packets actually start getting to the OnPacketReceive event handler 
   // only after setting the MAC filter to any value different from "0".
   HNAdapter.MacFilter := mfAll;

   HNConsole.printf('Adapter opened ...'#13#10#13#10,[]);
             
   Open_Btn.Enabled  := False;
   Close_Btn.Enabled := True;
 end;

//---------------------------------------------------------------------------
// Close_BtnClick - then adapter close button is pressed
//---------------------------------------------------------------------------
procedure TMainForm.Close_BtnClick(Sender: TObject);
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Set the MAC filter to "0" - now we do not receive packets
   HNAdapter.MacFilter := 0;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Stop the packet handling thread
   Thread.StopAndWait();
 end;

//---------------------------------------------------------------------------
// Stop - stop receiving packets
//---------------------------------------------------------------------------
procedure TMainForm.Stop();
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Set the MAC filter to "0" - now we do not receive packets
   HNAdapter.MacFilter := 0;

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Stop the timer statistics
   Timer.Enabled := False;   

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Refresh the statistics synchronously for the last time
   GetStat(False);

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Close the adapter
   HNAdapter.CloseAdapter();

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Clear the list of the busy packets.
   // It is necessary as at the moment of stopping the adapter and closing the 
   // adapter this list may contain unprocessed packets.
   // We close the list after closing the adapter, because no more packets
   // are added to the list.
   Thread.ClearBusyList();

   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Close the log file
   if(DumpToFile) then CloseHandle(hLogFile);

   Open_Btn.Enabled  := True;
   Close_Btn.Enabled := False;

   HNConsole.printf(#13#10'Adapter closed ...'#13#10#13#10,[]);
 end;

//---------------------------------------------------------------------------
// WndProc - the handler for the window messages
//---------------------------------------------------------------------------
procedure TMainForm.WndProc(var Msg: TMessage);
 begin
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // A window message from the working packet is received.
   // If the thread is stopped we stop intercepting the packets.
   if(Msg.Msg = HN_WM_THREAD_EXIT) then
    begin
      Stop(); // Stop intercepting the packets
    end;

   inherited WndProc(Msg);
 end;

//----------------------------------------------------------------------------------
// OnPacketReceive - a handler for the OnPacketReceive event of the HNAdapter object
//----------------------------------------------------------------------------------
procedure TMainForm.HNAdapterPacketRecv(Sender: TObject; pPacket: Pointer; PacketSize: Integer; pNumberID: PULargeInteger; pTimeStamp: PLargeInteger);
 var
  NumberID  : ULARGE_INTEGER;
  TimeStamp : LARGE_INTEGER;
 begin
   NumberID           := pNumberID^;
   TimeStamp.QuadPart := pTimeStamp^;
   //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // Add the recieved packet to the list for handling
   Thread.InsertPacket(pPacket,PacketSize,NumberID,TimeStamp);
 end;

procedure TMainForm.chkUseFastBpfClick(Sender: TObject);
begin
  HNAdapter.UseFastUserFilter := chkUseFastBpf.Checked;
end;

end.


