{****************************************************************************
**                                                                         **
**              Copyright (c) 2004 microOLAP Technologies LLC,             **
**                      Halturin A.P., Naumov D.A.                         **
**                              Source File                                **
**                                                                         **
****************************************************************************}

unit HNList;

interface
uses
  Windows;

type
  THNList = class
    protected
      m_List : LIST_ENTRY;
      Count  : Integer;

    public
      constructor Create; 
      destructor  Destroy; override;

    public
      procedure Clear();
      function  GetCount() : Integer;
      function  IsEmpty() : Boolean;

      procedure InsertHead(pItem : PListEntry); virtual;
      function  RemoveHead() : PListEntry; virtual;

      procedure InsertTail(pItem : PListEntry); virtual;
      function  RemoveTail() : PListEntry; virtual;

      function  Remove(pItem : PListEntry) : PListEntry; virtual;

      function  GetHeadPosition() : PListEntry; 
      function  GetNext(pPos : PListEntry) : PListEntry;
      function  GetPrev(pPos : PListEntry) : PListEntry;
  end;

implementation

//---------------------------------------------------------------------------
//  VOID InitializeListHead(PLIST_ENTRY ListHead);
procedure InitializeListHead(pListHead : PListEntry);
 begin
   pListHead^.Flink := pListHead;
   pListHead^.Blink := pListHead;
 end;

//  VOID InsertHeadList(PLIST_ENTRY ListHead,PLIST_ENTRY Entry);
procedure InsertHeadList(pListHead : PListEntry; pEntry : PListEntry);
 var
   _EX_Flink    : PListEntry;
   _EX_ListHead : PListEntry;
 begin
    _EX_ListHead        := pListHead;
    _EX_Flink           := _EX_ListHead^.Flink;
    pEntry^.Flink       := _EX_Flink;
    pEntry^.Blink       := _EX_ListHead;
    _EX_Flink^.Blink    := pEntry;
    _EX_ListHead^.Flink := pEntry;
 end;

//  VOID  RemoveEntryList(PLIST_ENTRY Entry);
procedure RemoveEntryList(pEntry : PListEntry);
 var
   _EX_Blink : PListEntry;
   _EX_Flink : PListEntry;
 begin
   _EX_Flink        := pEntry^.Flink;
   _EX_Blink        := pEntry^.Blink;
   _EX_Blink^.Flink := _EX_Flink;
   _EX_Flink^.Blink := _EX_Blink;
 end;

//  BOOLEAN IsListEmpty(PLIST_ENTRY ListHead);
function IsListEmpty(pListHead : PListEntry) : Boolean;
 begin
   if(pListHead^.Flink = pListHead) then Result := True
                                    else Result := False;
 end;

//  PLIST_ENTRY RemoveHeadList(PLIST_ENTRY ListHead);
function RemoveHeadList(pListHead : PListEntry) : PListEntry;
 begin
   Result := pListHead^.Flink;
   RemoveEntryList(pListHead^.Flink);
 end;

//  VOID InsertTailList(PLIST_ENTRY ListHead,PLIST_ENTRY Entry);
procedure InsertTailList(pListHead : PListEntry; pEntry : PListEntry);
 var
   _EX_Blink    : PListEntry;
   _EX_ListHead : PListEntry;
 begin
    _EX_ListHead        := pListHead;
    _EX_Blink           := _EX_ListHead^.Blink;
    pEntry^.Flink       := _EX_ListHead;
    pEntry^.Blink       := _EX_Blink;
    _EX_Blink^.Flink    := pEntry;
    _EX_ListHead^.Blink := pEntry;
 end;

//  PLIST_ENTRY RemoveTailList(PLIST_ENTRY ListHead);
function RemoveTailList(pListHead : PListEntry) : PListEntry;
 begin
   Result := pListHead^.Blink;
   RemoveEntryList(pListHead^.Blink);
 end;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// THNList implementation
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
// Create
//---------------------------------------------------------------------------
constructor THNList.Create;
 begin
   inherited Create;
   Clear();
 end;

//---------------------------------------------------------------------------
// Destroy
//---------------------------------------------------------------------------
destructor THNList.Destroy;
 begin
   inherited Destroy;
 end;

//---------------------------------------------------------------------------
// Clear
//---------------------------------------------------------------------------
procedure THNList.Clear();
 begin
   Count := 0;
   InitializeListHead(@m_List);
 end;

//---------------------------------------------------------------------------
// GetCount
//---------------------------------------------------------------------------
function THNList.GetCount() : Integer;
 begin
   Result := Count;
 end;

//---------------------------------------------------------------------------
// GetCount
//---------------------------------------------------------------------------
function THNList.IsEmpty() : Boolean;
 begin
   Result := IsListEmpty(@m_List);
 end;

//---------------------------------------------------------------------------
// InsertHead
//---------------------------------------------------------------------------
procedure THNList.InsertHead(pItem : PListEntry);
 begin
   InsertHeadList(@m_List,pItem);
   Inc(Count);
 end;

//---------------------------------------------------------------------------
// RemoveHead
//---------------------------------------------------------------------------
function THNList.RemoveHead() : PListEntry;
 var
   pEntry : PListEntry;
 begin
   if(IsEmpty()) then
     begin
       Result := nil;
       Exit;
     end;
   pEntry := RemoveHeadList(@m_List);
   Dec(Count);
   Result := pEntry;
 end;
    
//---------------------------------------------------------------------------
// InsertTail
//---------------------------------------------------------------------------
procedure THNList.InsertTail(pItem : PListEntry);
 begin
   InsertTailList(@m_List,pItem);
   Inc(Count);
 end;

//---------------------------------------------------------------------------
// RemoveTail
//---------------------------------------------------------------------------
function THNList.RemoveTail() : PListEntry;
 var
   pEntry : PListEntry;
 begin
   if(IsEmpty()) then
     begin
       Result := nil;
       Exit;
     end;
   pEntry := RemoveTailList(@m_List);
   Dec(Count);
   Result := pEntry;
 end;

//---------------------------------------------------------------------------
// Remove
//---------------------------------------------------------------------------
function THNList.Remove(pItem : PListEntry) : PListEntry;
 begin
   if(IsEmpty()) then
     begin
       Result := nil;
       Exit;
     end;
   RemoveEntryList(pItem);
   Dec(Count);
   Result := pItem;
 end;

//---------------------------------------------------------------------------
// GetHeadPosition
//---------------------------------------------------------------------------
function THNList.GetHeadPosition() : PListEntry;
 begin
   if(m_List.Flink = @m_List) then Result := nil
                              else Result := m_List.Flink;
 end;

//---------------------------------------------------------------------------
// GetNext
//---------------------------------------------------------------------------
function THNList.GetNext(pPos : PListEntry) : PListEntry;
 begin
   if(pPos^.Flink = @m_List) then Result := nil
                             else Result := pPos^.Flink;
 end;

//---------------------------------------------------------------------------
// GetPrev
//---------------------------------------------------------------------------
function THNList.GetPrev(pPos : PListEntry) : PListEntry;
 begin
   if(pPos^.Blink = @m_List) then Result := nil
                             else Result := pPos^.Blink;
 end;

end.
