{****************************************************************************
**                                                                         **
**              Copyright (c) 2004 microOLAP Technologies LLC,             **
**                      Halturin A.P., Naumov D.A.                         **
**                              Source File                                **
**                                                                         **
****************************************************************************}

unit HNSyncList;

interface
uses
  Windows, HNList;

type
  THNSyncList = class(THNList)
    private
      ListCS : TRTLCriticalSection;
      
    public
      constructor Create;
      destructor  Destroy; override;

    public
      procedure InsertHead(pItem : PListEntry); override;
      function  RemoveHead() : PListEntry; override;

      procedure InsertTail(pItem : PListEntry); override;
      function  RemoveTail() : PListEntry; override;

      function  Remove(pItem : PListEntry) : PListEntry; override;

      procedure Lock();
      procedure Unlock();
  end;

implementation

//---------------------------------------------------------------------------
// Create
//---------------------------------------------------------------------------
constructor THNSyncList.Create;
 begin
   inherited Create;
   InitializeCriticalSection(ListCS);
 end;

//---------------------------------------------------------------------------
// Destroy
//---------------------------------------------------------------------------
destructor THNSyncList.Destroy;
 begin
   DeleteCriticalSection(ListCS);
   inherited Destroy;
 end;

//---------------------------------------------------------------------------
// Lock
//---------------------------------------------------------------------------
procedure THNSyncList.Lock();
 begin
   EnterCriticalSection(ListCS);
 end;

//---------------------------------------------------------------------------
// Unlock
//---------------------------------------------------------------------------
procedure THNSyncList.Unlock();
 begin
   LeaveCriticalSection(ListCS);
 end;

//---------------------------------------------------------------------------
// InsertHead
//---------------------------------------------------------------------------
procedure THNSyncList.InsertHead(pItem : PListEntry);
 begin
   Lock();
   inherited InsertHead(pItem);
   Unlock();
 end;

//---------------------------------------------------------------------------
// RemoveHead
//---------------------------------------------------------------------------
function THNSyncList.RemoveHead() : PListEntry;
 begin
   Lock();
   Result := inherited RemoveHead();
   Unlock();
 end;

//---------------------------------------------------------------------------
// InsertTail
//---------------------------------------------------------------------------
procedure THNSyncList.InsertTail(pItem : PListEntry);
 begin
   Lock();
   inherited InsertTail(pItem);
   Unlock();
 end;

//---------------------------------------------------------------------------
// RemoveTail
//---------------------------------------------------------------------------
function THNSyncList.RemoveTail() : PListEntry;
 begin
   Lock();
   Result := inherited RemoveTail();
   Unlock();
 end;

//---------------------------------------------------------------------------
// Remove
//---------------------------------------------------------------------------
function THNSyncList.Remove(pItem : PListEntry) : PListEntry;
 begin
   Lock();
   Result := inherited Remove(pItem);
   Unlock();
 end;

end.

