{****************************************************************************
**                                                                         **
**              Copyright (c) 2004 microOLAP Technologies LLC,             **
**                      Halturin A.P., Naumov D.A.                         **
**                              Source File                                **
**                                                                         **
****************************************************************************}
unit HNThread;

interface

uses
  Windows;

type
  THNThread = class
    protected
      hThread    : THandle;
      hStopEvent : THandle; // Signal state if thread stop

    public
      constructor Create;
      destructor  Destroy; override;

    public
      function  Start(Priority : Integer) : BOOL;
      function  Execute() : DWORD ; virtual;
      function  Resume()  : DWORD;
      function  Suspend() : DWORD;
      procedure StopAndWait();

  end;
  PHNThread = ^THNThread;


implementation

//---------------------------------------------------------------------------
// Create
//---------------------------------------------------------------------------
constructor THNThread.Create;
 begin
   inherited Create;
   hStopEvent := CreateEvent(nil,True,True,nil);
   hThread    := 0;
 end;

//---------------------------------------------------------------------------
// Destroy
//---------------------------------------------------------------------------
destructor THNThread.Destroy;
 begin
   StopAndWait();
   CloseHandle(hStopEvent);
   inherited Destroy;
 end;

//---------------------------------------------------------------------------
// Resume
//---------------------------------------------------------------------------
function THNThread.Resume()  : DWORD;
 begin
   Result := ResumeThread(hThread);
 end;

//---------------------------------------------------------------------------
// Suspend
//---------------------------------------------------------------------------
function THNThread.Suspend() : DWORD;
 begin
   Result := SuspendThread(hThread);
 end;

//---------------------------------------------------------------------------
// Execute
//---------------------------------------------------------------------------
function THNThread.Execute() : DWORD;
 begin
   Result := 0;
 end;

//---------------------------------------------------------------------------
// _raw_Execute
//---------------------------------------------------------------------------
function _raw_Execute(pParam : Pointer) : DWORD; stdcall;
 begin
   Result := (THNThread(pParam)).Execute();
   SetEvent((THNThread(pParam)).hStopEvent);
   (THNThread(pParam)).hThread := 0;
 end;

//---------------------------------------------------------------------------
// Start
//---------------------------------------------------------------------------
function THNThread.Start(Priority : Integer) : BOOL;
 var
   ThreadID : DWORD;
 begin
   hThread := CreateThread(nil,0,@_raw_Execute,Self,CREATE_SUSPENDED,ThreadID);
   if(hThread <> 0) then
    begin
      ResetEvent(hStopEvent);
      SetThreadPriority(hThread,Priority);
      Resume();
    end;
   Result := (hThread <> 0);
 end;

//---------------------------------------------------------------------------
// StopAndWait
//---------------------------------------------------------------------------
procedure THNThread.StopAndWait();
 begin
   if(hThread <> 0) then
    begin
      SetEvent(hStopEvent);
      WaitForSingleObject(hThread,INFINITE);
      hThread := 0;
    end;
 end;

end.

